#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# This script deletes files using wipeStudy method.
# Provide an accession of a study/template which needs to be deleted.

from __future__ import print_function, unicode_literals

import re
import sys

from odm_sdk import GenestackServerException
from odm_sdk.scripts.utils import colored, GREEN, RED
from odm_sdk.utils import make_connection_parser, get_connection


def main():
    parser = make_connection_parser()
    group = parser.add_argument_group('required arguments')
    group.add_argument('--accession', metavar='<accession>',
                       help='accession of a study/template to delete', required=True)
    args = parser.parse_args()
    connection = get_connection(args)

    accession = args.accession
    try:
        connection.application('genestack/study-metainfo-editor').invoke('wipeStudy', accession)
        print(colored("Success", GREEN))
    except GenestackServerException as e:
        message = e
        if e.stack_trace is not None:
            p = re.compile(r"GenestackRestApiException: (.*?)(?=\n)")
            result = p.search(e.stack_trace)
            if result:
                message = result.group(1)
        print(colored(message, RED), file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import print_function

import sys

from odm_sdk import Metainfo
from odm_sdk.scripts.utils import colored, RED, YELLOW

ACCESSION = {
    "isRequired": True,
    "metainfoType": "com.genestack.api.metainfo.StringValue",
    "name": Metainfo.ACCESSION,
    "isReadOnly": True,
    "description": "Genestack-generated accession"
}
DATA_CLASS = {
    "isRequired": True,
    "metainfoType": "com.genestack.api.metainfo.StringValue",
    "name": Metainfo.DATA_CLASS,
    "isReadOnly": True,
    "description": "Technical field specifying the exact type of data you have uploaded."
}
FEATURES_STRING = {
    "isRequired": False,
    "metainfoType": "com.genestack.api.metainfo.StringValue",
    "name": Metainfo.FEATURES_STRING,
    "isReadOnly": True,
    "description": "If your dataset contains multiple columns for feature, "
                   "this parameter lists columns with text content (e.g., Gene Name)."
}
FEATURES_NUMERIC = {
    "isRequired": False,
    "metainfoType": "com.genestack.api.metainfo.StringValue",
    "name": Metainfo.FEATURES_NUMERIC,
    "isReadOnly": True,
    "description": "Technical field for dataset containing multiple columns for feature, "
                   "this parameter lists columns with numeric content "
                   "(e.g., Retention Time, M/Z ratio)."
}
VALUES_NUMERIC = {
    "isRequired": False,
    "metainfoType": "com.genestack.api.metainfo.StringValue",
    "name": Metainfo.VALUES_NUMERIC,
    "isReadOnly": True,
    "description": "Technical field for dataset containing multiple measurement or value types "
                   "for each item (like a sample, library, or preparation), "
                   "this parameter lists these (e.g. Intensity, Fold Change, p-value)."
}

__required_data_types = {
    Metainfo.ACCESSION: [],  # empty list means that this field is required for all data types
    Metainfo.DATA_CLASS: [
        'genestack:transcriptomicsParent',
        'genestack:genomicsParent',
        'genestack:facsParent'
    ],
    Metainfo.FEATURES_STRING: ['genestack:transcriptomicsParent'],
    Metainfo.FEATURES_NUMERIC: ['genestack:transcriptomicsParent'],
    Metainfo.VALUES_NUMERIC: ['genestack:transcriptomicsParent']
}

__items_by_name = {
    Metainfo.ACCESSION: ACCESSION,
    Metainfo.DATA_CLASS: DATA_CLASS,
    Metainfo.FEATURES_STRING: FEATURES_STRING,
    Metainfo.FEATURES_NUMERIC: FEATURES_NUMERIC,
    Metainfo.VALUES_NUMERIC: VALUES_NUMERIC
}

__item_positions_by_name = {
    Metainfo.ACCESSION: 0,
    Metainfo.DATA_CLASS: 1,
    # -1 means that the field should be added at the end of the list
    Metainfo.FEATURES_STRING: -1,
    Metainfo.FEATURES_NUMERIC: -1,
    Metainfo.VALUES_NUMERIC: -1
}


def validate_content(content):
    for item in content:
        field_name = item['name']
        data_type = item['dataType']
        data_types = __required_data_types.get(field_name)
        if data_types is not None and _contains_data_type(data_types, data_type):
            technical_item = _with_data_type(__items_by_name[field_name], data_type)
            if _equals_ignoring_description(item, technical_item):
                print(colored(f'Template field "{field_name}" of type "{data_type}" '
                              f'is predefined and can be omitted.', YELLOW))
            else:
                print(colored(f'Template field "{field_name}" of type "{data_type}" '
                              f'is predefined and cannot be edited.', RED), file=sys.stderr)
                sys.exit(1)


def enrich_items(data_type, data_type_items):
    copied_items = data_type_items.copy()
    for field_name, data_types in __required_data_types.items():
        if _contains_data_type(data_types, data_type):
            technical_item = _with_data_type(__items_by_name[field_name], data_type)
            if not any(_equals_ignoring_description(item, technical_item)
                       for item in data_type_items):
                position = __item_positions_by_name[field_name]
                if position != -1:
                    copied_items.insert(position, technical_item)
                else:
                    copied_items.append(technical_item)
    return copied_items


def _contains_data_type(data_types, data_type):
    return len(data_types) == 0 or data_type in data_types


def _with_data_type(field_to_copy, data_type):
    new_field = field_to_copy.copy()
    new_field['dataType'] = data_type
    return new_field


def _equals_ignoring_description(item_fst, item_snd):
    fst_copy = item_fst.copy()
    snd_copy = item_snd.copy()
    fst_copy.pop('description', None)
    snd_copy.pop('description', None)
    return fst_copy == snd_copy



import unittest
from unittest.mock import patch

from odm_sdk import Metainfo
from odm_sdk.scripts.metainfo_templates.importers.template_technical_fields import (
    validate_content, enrich_items, ACCESSION, DATA_CLASS)


class TestTemplateTechnicalFields(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.t_data_type = 'genestack:transcriptomicsParent'
        cls.t_data_class = DATA_CLASS.copy()
        cls.t_data_class['dataType'] = cls.t_data_type

    @patch('sys.stderr.write')
    @patch('sys.exit')
    def test_exit_on_content_with_incomplete_accession(self, mock_sys_exit, mock_stderr_write):
        content = [
            {'name': Metainfo.ACCESSION, 'dataType': self.t_data_type},
            self.t_data_class,
            {
                'dictionaryName': 'Sequencing Platforms',
                'dataType': self.t_data_type,
                'isRequired': False,
                'metainfoType': 'com.genestack.api.metainfo.StringValue',
                'name': 'Experimental Platform',
                'isReadOnly': False
            }
        ]
        validate_content(content)
        mock_stderr_write.assert_called()
        self.assertEqual(mock_stderr_write.call_args_list[0][0][0],
                         'Template field "genestack:accession" of type '
                         '"genestack:transcriptomicsParent" is predefined and cannot be edited.')
        mock_sys_exit.assert_called_once()

    @patch('sys.stdout.write')
    @patch('sys.exit')
    def test_message_on_content_with_redundant_data_class(self, mock_sys_exit, mock_stdout_write):
        content = [
            {'name': 'custom field', 'dataType': self.t_data_type},
            self.t_data_class
        ]
        validate_content(content)
        mock_stdout_write.assert_called()
        self.assertEqual(mock_stdout_write.call_args_list[0][0][0],
                         'Template field "Data Class" of type '
                         '"genestack:transcriptomicsParent" is predefined and can be omitted.')
        mock_sys_exit.assert_not_called()

    @patch('sys.stdout.write')
    @patch('sys.exit')
    def test_message_on_content_with_different_description(self, mock_sys_exit, mock_stdout_write):
        t_accession = ACCESSION.copy()
        t_accession['dataType'] = self.t_data_type
        t_accession['description'] = 'custom old description'
        content = [t_accession]
        validate_content(content)
        mock_stdout_write.assert_called()
        self.assertEqual(mock_stdout_write.call_args_list[0][0][0],
                         'Template field "genestack:accession" of type '
                         '"genestack:transcriptomicsParent" is predefined and can be omitted.')
        mock_sys_exit.assert_not_called()

    @patch('sys.stderr.write')
    @patch('sys.stdout.write')
    def test_validation_passes_on_custom_content(self, mock_stdout_write, mock_stderr_write):
        content = [
            {'name': 'custom field', 'dataType': self.t_data_type},
            {'name': 'another field', 'dataType': self.t_data_type}
        ]
        validate_content(content)
        mock_stdout_write.assert_not_called()
        mock_stderr_write.assert_not_called()

    def test_all_technical_items_are_added_to_content(self):
        content = [
            {'name': 'custom field', 'dataType': self.t_data_type},
            {'name': 'another field', 'dataType': self.t_data_type}
        ]
        enriched_items = enrich_items(self.t_data_type, content)
        self.assertEqual(len(enriched_items), 7)
        self.assertEqual(enriched_items[0]['name'], Metainfo.ACCESSION)
        self.assertEqual(enriched_items[1]['name'], Metainfo.DATA_CLASS)
        self.assertEqual(enriched_items[2]['name'], 'custom field')
        self.assertEqual(enriched_items[3]['name'], 'another field')
        self.assertEqual(enriched_items[4]['name'], Metainfo.FEATURES_STRING)
        self.assertEqual(enriched_items[5]['name'], Metainfo.FEATURES_NUMERIC)
        self.assertEqual(enriched_items[6]['name'], Metainfo.VALUES_NUMERIC)

    def test_technical_items_are_filtered_by_data_type(self):
        data_type = 'genestack:genomicsParent'
        content = [
            {'name': 'custom field', 'dataType': data_type}
        ]
        enriched_items = enrich_items(data_type, content)
        self.assertEqual(len(enriched_items), 3)
        self.assertEqual(enriched_items[0]['name'], Metainfo.ACCESSION)
        self.assertEqual(enriched_items[1]['name'], Metainfo.DATA_CLASS)
        self.assertEqual(enriched_items[2]['name'], 'custom field')

    def test_technical_items_are_description_agnostic(self):
        t_accession = ACCESSION.copy()
        t_accession['dataType'] = self.t_data_type
        t_accession['description'] = 'custom old description'
        content = [t_accession]
        enriched_items = enrich_items(self.t_data_type, content)
        self.assertEqual(len(enriched_items), 5)
        filtered_items = [item for item in enriched_items if item['name'] == Metainfo.ACCESSION]
        self.assertEqual(len(filtered_items), 1)
        self.assertEqual(enriched_items[0]['description'], 'custom old description')

    def test_technical_items_are_not_duplicated(self):
        data_type = 'study'
        content = [
            {'name': 'custom field', 'dataType': data_type}
        ]
        enriched_items = enrich_items(data_type, content)
        self.assertEqual(len(enriched_items), 2)
        self.assertEqual(enriched_items[0]['name'], Metainfo.ACCESSION)
        self.assertEqual(enriched_items[1]['name'], 'custom field')
        secondly_enriched_items = enrich_items(data_type, enriched_items)
        self.assertEqual(secondly_enriched_items, enriched_items)


if __name__ == '__main__':
    unittest.main()



from odm_sdk import get_connection, Metainfo
from odm_sdk.scripts.dictionaries.load_init_share_dictionaries import (load_dictionary, sharing,
                                                                                   initialization)

def upload_disease():

    # Upload Disease Ontology
    disease_ontology = {
        'name': 'Disease Ontology - DOID',
        'url': 'https://odm-init.s3.amazonaws.com/dictionaries/odm-12611/doid.owl',
        'description': 'Classification of human diseases by etiology. Domain: Diseases / Human Health, Version: 2025-05-30. The original data is available via the link https://purl.obolibrary.org/obo/doid/releases/2025-05-30/doid.owl'
    }
    connection = get_connection()
    disease_ontology_accession = load_dictionary(connection, disease_ontology)
    sharing(connection, [disease_ontology_accession])
    initialization(connection, [disease_ontology_accession])

    # Upload Disease Ontology Extended with reference to Disease Ontology
    metainfo = Metainfo()
    metainfo.add_file_reference("genestack.dictionary:parent", disease_ontology_accession)
    disease_ontology_extended = {
        'name': 'Disease Ontology Extended',
        'url': 'https://odm-init.s3.amazonaws.com/dictionaries/odm-12611/disease_extension.csv',
        'description': 'Classification of human diseases by etiology enriched by term "Healthy" and its synonyms. Domain: Diseases / Human Health, Version: 2025-05-30, extended by term "Healthy". The original data is available via the link https://purl.obolibrary.org/obo/doid/releases/2025-05-30/doid.owl'
    }
    disease_ontology_extended_accession = load_dictionary(connection, disease_ontology_extended, metainfo=metainfo)
    sharing(connection, [disease_ontology_extended_accession])
    initialization(connection, [disease_ontology_extended_accession])


def upload_gene_ontology():

    gene_ontology = {
        "name": "Gene Ontology - GO",
        "url": "https://odm-init.s3.amazonaws.com/dictionaries/odm-12611/go.owl",
        "description": "Describes gene functions: biological process, molecular function, cellular component. Domain: Molecular Function / Biological Process, Version: 2024-11-03. The original data is available via the link https://purl.obolibrary.org/obo/go/releases/2024-11-03/extensions/go-plus.owl"
    }

    connection = get_connection()
    metainfo = Metainfo()
    metainfo.add_string(Metainfo.DATA_TYPE, 'Gene Ontology')
    gene_ontology_accession = load_dictionary(connection, gene_ontology, metainfo=metainfo)
    sharing(connection, [gene_ontology_accession])
    initialization(connection, [gene_ontology_accession])


def main():
    """
    Currently we don't support metainfo parsing via auxiliary scripts from json in proper way.
    In the future we have to add this possibility and rework script to using only CLI call.
    """
    upload_disease()
    upload_gene_ontology()


if __name__ == '__main__':
    main()



from odm_sdk import (Application, FileFilter, GenestackException, Metainfo,
                              validate_constant)

CALCULATE_CHECKSUMS_KEY = 'genestack.checksum:markedForTests'
EXPECTED_CHECKSUM_PREFIX = 'genestack.checksum.expected:'
FILE_BATCH_SIZE = 500


class SpecialFolders(object):
    """
    - ``IMPORTED``: folder with files created by Data Importers
    - ``CREATED``: folder with files created by ``Preprocess`` and ``Analyse`` applications
    - ``TEMPORARY``: folder with temporary files
    - ``UPLOADED``: folder with uploaded raw files
    - ``MY_DATASETS``: folder with created datasets
    """
    IMPORTED = 'imported'
    CREATED = 'created'
    TEMPORARY = 'temporary'
    UPLOADED = 'uploaded'
    MY_DATASETS = 'my datasets'


class SortOrder(object):
    """
    Sort orders for file search queries
    """
    BY_NAME = "BY_NAME"
    BY_ACCESSION = "BY_ACCESSION"
    BY_LAST_UPDATE = "BY_LAST_UPDATE"
    DEFAULT = "DEFAULT"


class FilesUtil(Application):
    """
    An application to perform file management operations on Genestack.
    """
    APPLICATION_ID = 'genestack/filesUtil'

    CONTAINER = 'com.genestack.api.files.IContainerFile'
    FOLDER = 'com.genestack.api.files.IFolder'
    EXPERIMENT = 'com.genestack.bio.files.IExperiment'
    DATASET = 'com.genestack.api.files.IDataset'

    FILE = 'com.genestack.api.files.IFile'
    UNALIGNED_READS = 'com.genestack.bio.files.IUnalignedReads'
    UNALIGNED_READS_DATA = 'com.genestack.bio.files.IUnalignedReadsData'
    ALIGNED_READS = 'com.genestack.bio.files.IAlignedReads'
    VARIATION_FILE = 'com.genestack.bio.files.IVariationFile'
    APPLICATION_PAGE_FILE = 'com.genestack.api.files.IApplicationPageFile'
    REFERENCE_GENOME = 'com.genestack.bio.files.IReferenceGenome'
    AUXILIARY_FILE = 'com.genestack.api.files.IAuxiliaryFile'
    INDEX_FILE = 'com.genestack.api.files.IIndexFile'
    CODON_TABLE = 'com.genestack.bio.files.ICodonTable'
    GENOME_BED_DATA = 'com.genestack.bio.files.genomedata.IGenomeBEDData'
    GENOME_WIGGLE_DATA = 'com.genestack.bio.files.genomedata.IGenomeWiggleData'
    GENOME_ANNOTATIONS = 'com.genestack.bio.files.IGenomeAnnotations'
    HTSEQ_COUNTS = 'com.genestack.bio.files.IHTSeqCounts'
    EXTERNAL_DATABASE = 'com.genestack.bio.files.IExternalDataBase'
    PREFERENCES_FILE = 'com.genestack.api.files.IPreferencesFile'
    REPORT_FILE = 'com.genestack.api.files.IReportFile'
    RAW_FILE = 'com.genestack.api.files.IRawFile'
    MICROARRAY_ASSAY = 'com.genestack.bio.files.IMicroarrayAssay'
    MICROARRAY_DATA = 'com.genestack.bio.files.IMicroarrayData'
    SEQUENCING_ASSAY = 'com.genestack.bio.files.ISequencingAssay'
    FEATURE_LIST = 'com.genestack.bio.files.IFeatureList'
    EXPRESSION_SIGNATURE = 'com.genestack.bio.files.IGeneExpressionSignature'
    EXPRESSION_LEVELS = 'com.genestack.bio.files.IExpressionLevels'

    MAX_FILE_SEARCH_LIMIT = 2000
    MAX_RELATED_TERMS_LIMIT = 10000

    def find_file_by_name(self, name, parent=None, file_class=FILE):
        """
        Finds file with specified name (ignore case!) and type.
        If no file is found ``None`` is returned.
        If more than one file is found the first one is returned.
        If the parent container is not found, the corresponding exceptions are thrown.

        :param name: file name
        :type name: str
        :param parent: parent accession, private folder is default
        :type parent: str
        :param file_class: File class to be returned, default IFile
        :type file_class: str
        :return: file accession
        :rtype: str
        """
        return self.invoke(
            'getFileByName',
            name, parent, file_class
        )

    def find_application_page_file_by_name(self, name, parent=None):
        return self.find_file_by_name(name, parent, self.APPLICATION_PAGE_FILE)

    def create_folder(self, name, parent=None, description=None, metainfo=None):
        """
        Create a folder.

        :param name: name of the folder
        :type name: str
        :param parent: if not specified, create folder in the user's private folder
        :type parent: str
        :param description: description of the folder (goes into the metainfo)
        :type description: str
        :param metainfo: additional :py:class:`~odm_sdk.Metainfo`.
            Description and accession should be specified either via arguments or in a metainfo object (but not in both).
        :type metainfo: Metainfo
        :return: accession of created folder
        """
        metainfo = metainfo or Metainfo()
        metainfo.add_string(Metainfo.NAME, name)
        if description is not None:
            metainfo.add_string(Metainfo.DESCRIPTION, description)
        return self.invoke('createFolder', parent, metainfo)

    def find_or_create_folder(self, name, parent=None):
        """
        Return the folder accession if it already exists, and create it otherwise.
        If more than one folder is found the first one is returned.

        :param name: display name
        :type name: str
        :param parent: parent accession, use home folder if None
        :type parent: str
        :return: accession of folder
        :rtype: str
        """
        return self.invoke('findOrCreateFolder', name, parent)

    def unlink_file(self, accession, parent):
        """
        Unlink a file from a folder.

        :param accession: file accession
        :type accession: str
        :param parent: folder accession
        :type parent: str
        :rtype: None
        """
        self.invoke('unlinkFiles', {accession: [parent]})

    def link_files(self, children_to_parents_dict):
        """
        Link files to containers.

        :param children_to_parents_dict: dictionary where keys are accessions of the files to link, and
            values are lists of accessions of the containers to link into
        :type: dict

        :rtype: None
        """
        self.invoke('linkFiles', children_to_parents_dict)

    def unlink_files(self, children_to_parents_dict):
        """
        Unlink files from containers.

        :param children_to_parents_dict: dictionary where keys are accessions of the files to unlink, and
            values are lists of accessions of the containers to unlink from
        :type children_to_parents_dict: dict[str, list[str]]
        :rtype: None
        """
        self.invoke('unlinkFiles', children_to_parents_dict)

    def add_metainfo_string_value(self, accession_list, key, value):
        """
        Add a string value to the metainfo of specified files.

        :param accession_list: list of files to be updated
        :type accession_list: list[str]
        :param key: metainfo key
        :type key: str
        :param value: string
        :type value: str
        :rtype: None
        """
        self.invoke('addMetainfoStringValue', accession_list, key, value)

    def add_metainfo_values(self, accession, metainfo, skip_existing_keys=True, replace_existing_keys=False):
        """
        Add metainfo to a specified file.
        By default, metainfo keys that are already present in the file will be skipped.

        :param accession: accession of the file to update
        :param metainfo: metainfo object containing the metainfo to add
        :type metainfo: Metainfo
        :param skip_existing_keys: ignore metainfo keys that are already present in the file's metainfo
            (default: ``True``)
        :type skip_existing_keys: bool
        :param replace_existing_keys: replace the existing metainfo value for the metainfo keys
            that are already present in the file's metainfo (default: ``False``)
        :type replace_existing_keys: bool
        :rtype: None
        """
        self.invoke('addMetainfoValues', accession, metainfo, skip_existing_keys, replace_existing_keys)

    def get_special_folder(self, name):
        """
        Return the accession of a special folder.

        Available special folders are described in :class:`.SpecialFolders`

        :param name: special folder name
        :type name: str
        :return: accession
        :rtype: str
        :raises: GenestackException: if folder name is unknown
        """
        special_folders = (SpecialFolders.IMPORTED, SpecialFolders.CREATED, SpecialFolders.TEMPORARY,
                           SpecialFolders.UPLOADED, SpecialFolders.MY_DATASETS)
        if name not in special_folders:
            raise GenestackException("Name '%s' must be one of %s" % (name, ', '.join(special_folders)))
        return self.invoke('getSpecialFolder', name)

    def get_group_folder_info(self, group_accession):
        raise NotImplementedError("FilesUtil.get_group_folder_info has been removed in v0.33")

    def get_folder(self, parent, *names, **kwargs):
        """
        Find a subfolder (by name) in a folder passed as an accession,
        returning accession of that subfolder.  If several names are provided,
        treat them as a path components for the sub-sub-...-folder down the
        folder hierarchy, returning accession of that deepmost folder:

        - ``fu.get_folder('GS777', 'RNASeq')`` looks for subfolder with *name*
          "RNASeq" in folder with *accession* "GS777", and returns accession of
          that "RNASeq" subfolder;

        - ``fu.get_folder('GS777', 'Experiments', 'RNASeq')`` looks for
          subfolder with *name* "Experiments" in a folder with *accession*
          "GS777", then looks for "RNASeq" in "Experiments", and returns the
          accession of "RNASeq".

        If ``create=True`` is passed as a *kwarg*, all the folders in ``names``
        hierarchy will be created (otherwise ``GenestackException`` is raised).

        :param parent: accession of folder to search in
        :type parent: str
        :param \*names: tuple of "path components", a hierarchy of folders to
                        find
        :type names: tuple
        :param create: whether to create folders from ``names`` if they don't
                       exist or not; default is ``False`` (raise
                       ``GenestackException`` if any folder doesn't exist)
        :type create: bool
        :return: accession of found (or created) subfolder
        :rtype: str
        :raises GenestackException:
            if no name is passed, or folder with required name is not found
            (and shouldn't be created)
        """
        if not names:
            raise GenestackException("At least one path should be specified")

        create = bool(kwargs.get('create'))
        for path in names:
            if create:
                parent = self.find_or_create_folder(path, parent=parent)
            else:
                _parent_accession = self.find_file_by_name(path, parent=parent, file_class=self.FOLDER)
                if _parent_accession is None:
                    raise GenestackException('Cannot find folder with name "%s" '
                                             'in folder with accession: %s'
                                             % (path, parent))
                parent = _parent_accession
        return parent

    def get_home_folder(self):
        """
        Return the accession of the current user's home folder.

        :return: accession of home folder
        :rtype: str
        """
        return 'private'

    def get_public_folder(self):
        """
        Return the accession of the ``Public`` folder on the current Genestack instance.

        :return: accession of ``Public`` folder
        :rtype: str
        """
        return 'public'

    def mark_for_tests(self, app_file):
        """
        Mark Genestack file as test one by adding corresponding key to metainfo.
        Test file will calculate md5 checksums of its encapsulated physical files
        during initialization.

        :param app_file: accession of file
        :return: None
        """
        metainfo = Metainfo()
        metainfo.add_boolean(CALCULATE_CHECKSUMS_KEY, True)
        self.add_metainfo_values(app_file, metainfo)

    def mark_obsolete(self, accession):
        """
        Mark Genestack file as obsolete one by adding corresponding key to metainfo.

        :param accession: accession of file
        :return: None
        """
        metainfo = Metainfo()
        metainfo.add_boolean('genestack:obsolete', True)
        self.add_metainfo_values(accession, metainfo)

    def add_checksums(self, app_file, expected_checksums):
        """
        Add expected MD5 checksum to the metainfo of a CLA file.
        Expected checksums are calculated in the following way:

            - The number of checksums equals number of entries in storage.
              For instance, a Reference Genome file has 2 entries (annotation and sequence files).
            - If there are multiple files in one entry, they will be concatenated in the same order
              as they were ``PUT`` to storage by the initialization script.
            - If a file is marked for testing, then after initialization its metainfo
              will contain both expected and actual checksum values.

        :param app_file: accession of application file
        :param expected_checksums: collection of MD5 checksums
        :return: None
        """
        metainfo = Metainfo()
        for key, value in expected_checksums.items():
            metainfo.add_string('%s%s' % (EXPECTED_CHECKSUM_PREFIX, key), value)
        self.add_metainfo_values(app_file, metainfo)

    def initialize(self, accessions):
        """
        Start initialization for the specified accessions.
        Missed accession and initialization failures are ignored silently.

        :param list[str] accessions: list of accessions
        :rtype: None
        """
        self.invoke('initialize', accessions)

    # TODO deprecate and use FilesUtil(self.connection).get_infos(accessions) instead
    def load_info(self, accessions):
        """
        Takes as input a list of file accessions and returns a list of dictionaries (one for each accession)
        with the following structure:

            - accession: (str) file accession
            - name: (str) file name if the file exists
            - status: (str) initialization status

        The possible values for ``status`` are:

            - NoSuchFile
            - NotApplicable
            - NotStarted
            - InProgress
            - Complete
            - Failed


        :param list[str] accessions: list of accessions
        :return: list of dictionaries
        :rtype: list
        """
        return self.invoke('loadInfo', accessions)

    def search_files(self, accession):
        return self.invoke('searchFiles',
                           accession,  # type: str
                           {}, [], 100, 0, None, False)
